.stat-card[data-v-5c4af34d]{background:var(--bg-surface);border:1px solid var(--border);border-radius:var(--radius-lg);padding:20px;display:flex;align-items:flex-start;gap:14px;transition:border-color var(--transition);box-shadow:var(--shadow-sm)}.stat-card[data-v-5c4af34d]:hover{border-color:var(--border-light)}.stat-card__icon[data-v-5c4af34d]{font-size:.85rem;font-weight:700;width:40px;height:40px;display:flex;align-items:center;justify-content:center;background:var(--accent-dim);color:var(--accent);border-radius:var(--radius-md);flex-shrink:0;font-family:var(--font-mono)}.stat-card--success .stat-card__icon[data-v-5c4af34d]{background:var(--success-dim);color:var(--success)}.stat-card--warning .stat-card__icon[data-v-5c4af34d]{background:var(--warning-dim);color:var(--warning)}.stat-card--error .stat-card__icon[data-v-5c4af34d]{background:var(--error-dim);color:var(--error)}.stat-card__value[data-v-5c4af34d]{font-size:1.5rem;font-weight:700;line-height:1.2}.stat-card__label[data-v-5c4af34d]{font-size:.8rem;color:var(--text-secondary);margin-top:2px}.stat-card__sub[data-v-5c4af34d]{font-size:.75rem;color:var(--text-muted);margin-top:4px}
