from typing import List

from fastapi import APIRouter

from ..base import BaseAPIAdapter


class AssistantAPIAdapter(BaseAPIAdapter):
    def __init__(self, prefix: str = ""):
        super().__init__(prefix)

    def get_routers(self) -> List[APIRouter]:
        from .assistant import router as assistant_router
        from .threads import router as threads_router
        from .message import router as message_router
        from .run import router as run_router
        from .run_step import router as run_step_router
        return [assistant_router, run_router, threads_router, message_router, run_step_router]
