.compare-grid[data-v-f621b5a1]{display:grid;gap:16px}.compare-col[data-v-f621b5a1]{display:flex;flex-direction:column;gap:0}.compare-col__header[data-v-f621b5a1]{display:flex;align-items:center;justify-content:space-between;margin-bottom:14px;padding-bottom:12px;border-bottom:1px solid var(--border)}.compare-section[data-v-f621b5a1]{margin-bottom:14px}.compare-meta>div[data-v-f621b5a1]{font-size:.825rem;padding:4px 0;display:flex;justify-content:space-between}.label[data-v-f621b5a1]{color:var(--text-muted);font-size:.75rem}
